namespace "kegg" do

  def parse_category_line(level, line)
    text  = line.sub(/^[A-Z]/, '').strip

    # <a href="/dbget-bin/www_bget?genome+hsa">hsa</a> -> "/dbget-...", "hsa"
    regexp = /<a\s+href=\"?([^\">]+)\"?>(.*)<\/a>/i
    href, anchor = text.match(regexp).to_a.values_at(1, 2)

    # clean up
    text.gsub!(/<a[^>]+>(.*)<\/a>/i, '')
    text.gsub!(/<b>(.*)<[+\/]b>/i, '\1')

    node, link, = text.split(/\t/)

    # 1.1 Carbohydrate Metabolism -> "1.1", "Carbohydrate Metabolism"
    node.strip!
    if node.match(/^[\d\.]+\s/)
      section, definition = node.split(/\s+/, 2)
    else
      section = ""
      definition = node
    end

    # [PATH:map00010] -> path:map00010
    if link
      target = link.tr('[]', '').sub(/[A-Z]+:/) {|db| db.downcase}
    end

    [href, anchor, section, definition, target].map! {|x| x.to_s.strip}

    hash = {
      :line => line,
      :level => level,
      :section => section,
      :definition => definition,
      :target => target,
      :href => href,
      :anchor => anchor,
    }
    return hash
  end

  desc "Load KEGG category (.keg) file"
  task :load_category => :environment do
    STDERR.puts "\n>>> begin (load_category)"

    ActiveRecord::Base.transaction do
      require 'kconv'
      $KCODE = "u"

      indent = Hash.new
      ("A".."Z").to_a.each_with_index do |c, i|
        indent[c] = i + 1
      end

      root = Category.root || Category.create(:level => "root")

      KEGG_CONFIG::CATEGORY_FILES.each do |filename|
        STDERR.puts filename
        hash = {
          :level => File.basename(filename),
        }
        parent = Category.create(hash)
        root.add_child(parent)

        stack = Array.new
        stack[0] = parent.id
        STDERR.puts stack.inspect

        File.open(filename).each do |line|
          line = line.toutf8

          if title = line[/^\#NAME\s+(\S+)/, 1]
            STDERR.puts title
            parent = Category.find(stack[0])
            hash = {
              :definition => title,
            }
            parent.update_attributes(hash)
          elsif heading = line[/^([A-Z])/, 1]
            next if line.strip.length == 1
            level = indent[heading]
            hash = parse_category_line(level, line)
            STDERR.puts hash.pretty_inspect
            child = Category.create(hash)
            parent = Category.find(stack[level-1])
            parent.add_child(child)
            stack[level] = child.id
            STDERR.puts stack[0, level+1].inspect
          end
        end
      end
    end

    STDERR.puts "\n>>> end (load_category)"
  end
end
