namespace "kegg" do
  desc "Load KEGG COMPOUND database"
  task :load_compound => :environment do
    STDERR.puts "\n>>> begin (load_compound)"

    ActiveRecord::Base.transaction do
      Bio::FlatFile.open(KEGG_CONFIG::COMPOUND_FILE).each do |entry|
        cpd = entry.entry_id
        STDERR.puts "\n#{cpd}"

        hash = {
          :entry     => cpd,
          :name      => entry.names,
          :formula   => entry.formula,
          :mass      => entry.mass,
          :comment   => entry.comment,
          :kcf       => entry.kcf,
        }
        Compound.create(hash)

        # linkdb
        entry.glycans.each do |name|
          save_link("cpd", cpd, "gl", name)
        end

        entry.reactions.each do |name|
          save_link("cpd", cpd, "rn", name)
        end

        entry.pathways.each do |line|
          if name = line[/PATH:\s*(\S+)/, 1]
            save_link("cpd", cpd, "path", name)
          end
        end

        entry.enzymes.each do |name|
          save_link("cpd", cpd, "ec", name)
        end

        entry.dblinks.each do |line|
          db, list = line.match(/([^:]+):\s*(\S+)/).to_a.values_at(1, 2)
          if db
            list.split(/\s+/).each do |name|
              save_link("cpd", cpd, db, name, "external")
            end
          end
        end
      end
    end

    STDERR.puts "\n>>> end (load_compound)"
  end
end
