namespace "kegg" do
  desc "Load KEGG ENZYME database"
  task :load_enzyme => :environment do
    STDERR.puts "\n>>> begin (load_enzyme)"

    ActiveRecord::Base.transaction do

      Bio::FlatFile.open(KEGG_CONFIG::ENZYME_FILE).each do |entry|
        ec = entry.entry_id
        STDERR.puts "\n#{ec}"

        hash = {
          :entry     => ec,
          :name      => entry.names,
          :sysname   => entry.sysname,
          :comment   => entry.comment,
          :reaction  => entry.reaction,
          :iubmb_reaction => entry.iubmb_reactions.join("; "),
          :kegg_reaction  => entry.kegg_reactions.join("; "),
        }
        Enzyme.create(hash)

        # TODO:
        #   classes
        #   references

        # linkdb
        entry.all_reac.scan(/R\d+/).each do |name|
          save_link("ec", ec, "rn", name)
        end

        entry.substrates.each do |compound|
          cpd = compound[/CPD:(\S+)\]/, 1]
          save_link("ec", ec, "cpd", cpd, "substrate")
        end

        entry.products.each do |compound|
          cpd = compound[/CPD:(\S+)\]/, 1]
          save_link("ec", ec, "cpd", cpd, "product")
        end

        entry.cofactors.each do |compound|
          cpd = compound[/CPD:(\S+)\]/, 1]
          save_link("ec", ec, "cpd", cpd, "cofactor")
        end

        entry.inhibitors.each do |compound|
          cpd = compound[/CPD:(\S+)\]/, 1]
          save_link("ec", ec, "cpd", cpd, "inhibitor")
        end

        entry.pathways.each do |line|
          if name = line[/PATH:\s*(\S+)/, 1]
            save_link("ec", ec, "path", name)
          end
        end

        entry.orthologs.each do |line|
          if name = line[/KO:\s*(\S+)/, 1]
            save_link("ec", ec, "ko", name)
          end
        end

        entry.genes.each do |line|
          db, genes = line.match(/(\S+):\s*(\S+)/).to_a.values_at(1, 2)
          if db
            genes.split(/\s/).each do |gene|
              org = db.downcase
              name = gene.sub(/\(.*/,'')
              save_link("ec", ec, org, name, "genes")
            end
          end
        end

        entry.diseases.each do |line|
          if name = line[/MIM:\s*(\S+)/, 1]
            save_link("ec", ec, "omim", name)
          end
        end

        entry.motifs.each do |line|
          dbname, name = line.match(/(\S+):\s*(\S+)/).to_a.values_at(1, 2)
          if dbname
            case dbname
            when "PS"
              db = "ps"
            else
              db = dbname.downcase
            end
            save_link("ec", ec, db, name, "motif")
          end
        end

        entry.structures.each do |name|
          save_link("ec", ec, "pdb", name)
        end

        entry.dblinks.each do |line|
          db, name = line.match(/([^:]+):\s*(\S+)/).to_a.values_at(1, 2)
          if db
            save_link("ec", ec, db, name, "external")
          end
        end
      end
    end
    STDERR.puts "\n>>> end (load_enzyme)"
  end
end
