namespace "kegg" do
  #
  # Note: use this after 'rake kegg:load_genome'
  #
  desc "Load KEGG GENES database"
  task :load_genes => :environment do
    STDERR.puts "\n>>> begin (load_genes)"

    KEGG_CONFIG::GENES_FILES.each do |genes_file|
      org = false

      ActiveRecord::Base.transaction do
        Bio::FlatFile.open(genes_file).each do |entry|
          name = File.basename(genes_file, ".ent")
        
          unless org
            genome = Genome.find(:first, :conditions => "name like '%#{name}%'")
            org = genome.entry
          end

          gene = entry.entry_id
          STDERR.puts "\n#{org}:#{gene}"

          hash = {
            :org        => org,
            :entry      => gene,
            :molecule   => entry.division,
            :name       => entry.name,
            :definition => entry.definition,
            :position   => entry.position,
            :aaseq      => entry.aaseq,
            :aalen      => entry.aalen,
            :ntseq      => entry.naseq,
            :ntlen      => entry.nalen,
          }
          Gene.create(hash)

          # dblinks
          entry.pathways.each do |line|
            if line[/^(\S+)$/]  	     		# bioruby release
              name = line
              save_link(org, gene, "path", name)
            elsif name = line[/PATH:\s*(\S+)/, 1]	# bioruby development
              save_link(org, gene, "path", name)
            end
          end

          entry.orthologs.each do |line|
            if name = line[/KO:\s*(\S+)/, 1]
              save_link(org, gene, "ko", name)
            end
          end

          entry.motif.each do |dbname, list|
            case dbname
            when "Pfam"
              db = "pf"
            when "PROSITE"
              db = "ps"
            end
            list.each do |name|
              save_link(org, gene, db, name, "motif")
            end
          end

          entry.dblinks.each do |db, list|
            list.each do |name|
              save_link(org, gene, db, name, "external")
            end
          end
        end
      end
    end

    STDERR.puts "\n>>> end (load_genes)"
  end
end

