namespace "kegg" do
  desc "Load KEGG GLYCAN database"
  task :load_glycan => :environment do
    STDERR.puts "\n>>> begin (load_glycan)"

    ActiveRecord::Base.transaction do
      Bio::FlatFile.open(KEGG_CONFIG::GLYCAN_FILE).each do |entry|
        gl = entry.entry_id
        STDERR.puts "\n#{gl}"

        hash = {
          :entry       => gl,
          :name        => entry.name,
          :composition => entry.fetch('COMPOSITION'),
          :mass        => entry.fetch('MASS'),
          :keggclass   => entry.keggclass,
          :remark      => entry.remark,
          :comment     => entry.comment,
          :kcf         => entry.kcf,
        }
        Glycan.create(hash)

        # linkdb
        entry.compounds.each do |name|
          save_link("gl", gl, "cpd", name)
        end

        entry.reactions.each do |name|
          save_link("gl", gl, "rn", name)
        end

        entry.pathways.each do |line|
          if name = line[/PATH:\s*(\S+)/, 1]
            save_link("gl", gl, "path", name)
          end
        end

        entry.enzymes.each do |name|
          save_link("gl", gl, "ec", name)
        end

        entry.orthologs.each do |line|
          if name = line[/KO:\s*(\S+)/, 1]
            save_link("gl", gl, "ko", name)
          end
        end

        entry.dblinks.each do |line|
          db, list = line.match(/([^:]+):\s*(\S+)/).to_a.values_at(1, 2)
          if db
            list.split(/\s+/).each do |name|
              save_link("gl", gl, db, name, "external")
            end
          end
        end

        # TODO:
        #   composition - hash?
        #   mass - node?
        #   keggclass - name?
        #   reference
      end
    end

    STDERR.puts "\n>>> end (load_glycan)"
  end
end
