namespace "kegg" do
  desc "Load KEGG PATHWAY database from KGML"
  task :load_pathway => :environment do
    STDERR.puts "\n>>> begin (load_pathway)"

    KEGG_CONFIG::KGML_ORG_DIRS.each do |kgml_org_dir|
      kgml_files = Dir.glob("#{kgml_org_dir}/*.xml").sort

      kgml_files.each do |kgml_file|
        kgml = Bio::KEGG::KGML.new(File.read(kgml_file))

        path = kgml.name.sub(/path:/, '')
        STDERR.puts "\n#{kgml.name}"

        ActiveRecord::Base.transaction do

          pathway = Pathway.new
          pathway.entry  = path
          pathway.org    = kgml.org
          pathway.number = kgml.number
          pathway.title  = kgml.title
          pathway.save

          kgml.entries.each do |entry|
            if entry.shape.nil? or entry.shape.empty?
              next 
            end

            if entry.link
              href = entry.link.sub(%r[http://www.genome.jp/], '')
            else
              href = nil
            end

            graphic = Graphic.new
            graphic.pathway  = pathway
            graphic.entry    = entry.entry_id
            graphic.name     = entry.name
            graphic.category = entry.category
            graphic.reaction = entry.reaction
            graphic.link     = href
            graphic.fgcolor  = entry.fgcolor
            graphic.bgcolor  = entry.bgcolor
            graphic.shape    = entry.shape
            graphic.x        = entry.x
            graphic.y        = entry.y
            graphic.width    = entry.width
            graphic.height   = entry.height
            graphic.save

            # linkdb - TODO: may need to check duplications
            case entry.category
            when "gene"
              entry.names.each do |dbentry|
                org, gene = dbentry.split(':')
                save_link("path", path, org, gene, graphic.entry)
              end
            when "compound"
              entry.names.each do |dbentry|
                mol, name = dbentry.split(':')
                case mol
                when "cpd"
                  db = mol
                when "glycan"
                  db = "gl"
                when "drug"	# only found in map04150 (drug:D00944)
                  db = "dr"
                else
                  STDERR.puts "### Unknown molecule '#{mol}' (#{kgml_file})"
                end
                save_link("path", path, mol, name, graphic.entry)
              end
            when "enzyme"
              entry.names.each do |dbentry|
                db, name = dbentry.split(':')
                save_link("path", path, "ec", name, graphic.entry)
              end
            when "ortholog"
              entry.names.each do |dbentry|
                db, name = dbentry.split(':')
                save_link("path", path, "ko", name, graphic.entry)
              end
            when "map"
              entry.names.each do |dbentry|
                db, name = dbentry.split(':')
                save_link("path", path, "path", name, graphic.entry)
              end

            # do nothing for now
            when "other"		# "ath:", "up:" etc.
            when "group"
            when "maplink"
            when "ECrel"
            when "GErel"
            when "PCrel"
            when "PPRel"		# only found in map04730
            when "PPrel"
            else
              STDERR.puts "### Unknown type '#{entry.category}' (#{kgml_file})"
            end
          end
        end
      end
    end

    STDERR.puts "\n>>> end (load_pathway)"
  end
end
